@@ -55,22 +55,15 @@
 #include "gtkmenuitem.h"
 #include "gtkmain.h"
 #include "gtksettings.h"
-#include "gtkintl.h"
 #include "gtkimage.h"
 #include "gtkstock.h"
 #include "gtkaccessible.h"
+#include "gtkprivate.h"
+#include "gtkintl.h"
 #include "gtkalias.h"
 
 #include <string.h>
 
-#ifdef GDK_WINDOWING_X11
-#include <X11/Xlib.h>
-#include "x11/gdkx.h"
-#elif defined GDK_WINDOWING_WIN32
-#define WIN32_LEAN_AND_MEAN
-#include <windows.h>
-#endif
-
 /* Number of elements in the custom palatte */
 #define GTK_CUSTOM_PALETTE_WIDTH 10
 #define GTK_CUSTOM_PALETTE_HEIGHT 2
@@ -146,6 +139,8 @@ struct _ColorSelectionPrivate
 
   /* Window for grabbing on */
   GtkWidget *dropper_grab_widget;
+  guint32    grab_time;
+  gboolean   has_grab;
 
   /* Connection to settings */
   gulong settings_connection;
@@ -169,11 +164,14 @@ static void gtk_color_selection_get_property    (GObject                 *object
 static void gtk_color_selection_realize         (GtkWidget               *widget);
 static void gtk_color_selection_unrealize       (GtkWidget               *widget);
 static void gtk_color_selection_show_all        (GtkWidget               *widget);
+static gboolean gtk_color_selection_grab_broken (GtkWidget               *widget,
+						 GdkEventGrabBroken      *event);
 
 static void     gtk_color_selection_set_palette_color   (GtkColorSelection *colorsel,
                                                          gint               index,
                                                          GdkColor          *color);
-static GdkGC   *get_focus_gc                            (GtkWidget         *drawing_area,
+static void    set_focus_line_attributes                (GtkWidget         *drawing_area,
+							 cairo_t           *cr,
 							 gint              *focus_width);
 static void     default_noscreen_change_palette_func    (const GdkColor    *colors,
 							 gint               n_colors);
@@ -224,7 +222,7 @@ static const guchar dropper_mask[] = {
 #define SAMPLE_HEIGHT 28
 
 static void color_sample_draw_sample (GtkColorSelection *colorsel, int which);
-static void color_sample_draw_samples (GtkColorSelection *colorsel);
+static void color_sample_update_samples (GtkColorSelection *colorsel);
 
 static void
 set_color_internal (GtkColorSelection *colorsel,
@@ -385,11 +383,9 @@ static void
 color_sample_draw_sample (GtkColorSelection *colorsel, int which)
 {
   GtkWidget *da;
-  gint x, y, i, wid, heig, f, n, goff;
-  guchar c[3 * 2], cc[3 * 4], *cp = c;
-  gdouble o;
-  guchar *buf;
+  gint x, y, wid, heig, goff;
   ColorSelectionPrivate *priv;
+  cairo_t *cr;
   
   g_return_if_fail (colorsel != NULL);
   priv = colorsel->private_data;
@@ -397,81 +393,74 @@ color_sample_draw_sample (GtkColorSelection *colorsel, int which)
   g_return_if_fail (priv->sample_area != NULL);
   if (!GTK_WIDGET_DRAWABLE (priv->sample_area))
     return;
-  
+
   if (which == 0)
     {
       da = priv->old_sample;
-      for (n = 0, i = COLORSEL_RED; n < 3; n++, i++)
-	c[n] = (guchar) (UNSCALE (priv->old_color[i]) >> 8);
       goff = 0;
     }
   else
     {
       da = priv->cur_sample;
-      for (n = 0, i = COLORSEL_RED; n < 3; n++, i++)
-	c[n] = (guchar) (UNSCALE (priv->color[i]) >> 8);
       goff =  priv->old_sample->allocation.width % 32;
     }
+
+  cr = gdk_cairo_create (da->window);
   
   wid = da->allocation.width;
   heig = da->allocation.height;
-  
-  buf = g_new (guchar, 3 * wid * heig);
-  
-#if 0
-  i = COLORSEL_RED;
-  for (n = 0; n < 3; n++)
-    {
-      c[n] = (guchar) (255.0 * priv->old_color[i]);
-      c[n + 3] = (guchar) (255.0 * priv->color[i++]);
-    }
-#endif
+
+  /* Below needs tweaking for non-power-of-two */  
+#define CHECK_SIZE 16  
   
   if (priv->has_opacity)
     {
-      o = (which) ? priv->color[COLORSEL_OPACITY] : priv->old_color[COLORSEL_OPACITY];
-      
-      for (n = 0; n < 3; n++)
-	{
-	  cc[n] = (guchar) ((1.0 - o) * 192 + (o * (gdouble) c[n]));
-	  cc[n + 3] = (guchar) ((1.0 - o) * 128 + (o * (gdouble) c[n]));
-	}
-      cp = cc;
+      /* Draw checks in background */
+
+      cairo_set_source_rgb (cr, 0.5, 0.5, 0.5);
+      cairo_rectangle (cr, 0, 0, wid, heig);
+      cairo_fill (cr);
+
+      cairo_set_source_rgb (cr, 0.75, 0.75, 0.75);
+      for (x = goff & -CHECK_SIZE; x < goff + wid; x += CHECK_SIZE)
+	for (y = 0; y < heig; y += CHECK_SIZE)
+	  if ((x / CHECK_SIZE + y / CHECK_SIZE) % 2 == 0)
+	    cairo_rectangle (cr, x - goff, y, CHECK_SIZE, CHECK_SIZE);
+      cairo_fill (cr);
     }
-  
-  i = 0;
-  for (y = 0; y < heig; y++)
+
+  if (which == 0)
     {
-      for (x = 0; x < wid; x++)
-	{
-	  if (priv->has_opacity)
-	    f = 3 * ((((goff + x) % 32) < 16) ^ ((y % 32) < 16));
-	  else
-	    f = 0;
-	  
-	  for (n = 0; n < 3; n++)
-	    buf[i++] = cp[n + f];
-	}
+      cairo_set_source_rgba (cr,
+			     priv->old_color[COLORSEL_RED], 
+			     priv->old_color[COLORSEL_GREEN], 
+			     priv->old_color[COLORSEL_BLUE],
+			     priv->has_opacity ?
+			        priv->old_color[COLORSEL_OPACITY] : 1.0);
     }
-  
-  gdk_draw_rgb_image (da->window,
-		      da->style->black_gc,
-		      0, 0,
-		      wid, heig,
-		      GDK_RGB_DITHER_NORMAL,
-		      buf,
-		      3*wid);
-  
-  
-  g_free (buf);
+  else
+    {
+      cairo_set_source_rgba (cr,
+			     priv->color[COLORSEL_RED], 
+			     priv->color[COLORSEL_GREEN], 
+			     priv->color[COLORSEL_BLUE],
+			     priv->has_opacity ?
+			       priv->color[COLORSEL_OPACITY] : 1.0);
+    }
+
+  cairo_rectangle (cr, 0, 0, wid, heig);
+  cairo_fill (cr);
+
+  cairo_destroy (cr);
 }
 
 
 static void
-color_sample_draw_samples (GtkColorSelection *colorsel)
+color_sample_update_samples (GtkColorSelection *colorsel)
 {
-  color_sample_draw_sample (colorsel, 0);
-  color_sample_draw_sample (colorsel, 1);
+  ColorSelectionPrivate *priv = colorsel->private_data;
+  gtk_widget_queue_draw (priv->old_sample);
+  gtk_widget_queue_draw (priv->cur_sample);
 }
 
 static gboolean
@@ -622,31 +611,41 @@ palette_paint (GtkWidget    *drawing_area,
 	       GdkRectangle *area,
 	       gpointer      data)
 {
+  cairo_t *cr;
+  gint focus_width;
+    
   if (drawing_area->window == NULL)
     return;
 
-  gdk_draw_rectangle (drawing_area->window,
-                      drawing_area->style->bg_gc[GTK_STATE_NORMAL],
-                      TRUE,
-                      area->x, area->y, area->width, area->height);
+  cr = gdk_cairo_create (drawing_area->window);
+
+  gdk_cairo_set_source_color (cr, &drawing_area->style->bg[GTK_STATE_NORMAL]);
+  gdk_cairo_rectangle (cr, area);
+  cairo_fill (cr);
   
   if (GTK_WIDGET_HAS_FOCUS (drawing_area))
     {
-      gint focus_width;
-      GdkGC *gc = get_focus_gc (drawing_area, &focus_width);
-      gdk_draw_rectangle (drawing_area->window,
-			  gc, FALSE, focus_width / 2, focus_width / 2,
-			  drawing_area->allocation.width - focus_width,
-			  drawing_area->allocation.height - focus_width);
-      g_object_unref (gc);
+      set_focus_line_attributes (drawing_area, cr, &focus_width);
+      g_print ("%g %g %g %g\n",
+	       focus_width / 2., focus_width / 2.,
+	       (double)drawing_area->allocation.width - focus_width,
+	       (double)drawing_area->allocation.height - focus_width);
+
+      cairo_rectangle (cr,
+		       focus_width / 2., focus_width / 2.,
+		       drawing_area->allocation.width - focus_width,
+		       drawing_area->allocation.height - focus_width);
+      cairo_stroke (cr);
     }
+
+  cairo_destroy (cr);
 }
 
-static GdkGC *
-get_focus_gc (GtkWidget *drawing_area,
-	      gint      *focus_width)
+static void
+set_focus_line_attributes (GtkWidget *drawing_area,
+			   cairo_t   *cr,
+			   gint      *focus_width)
 {
-  GdkGC *gc = gdk_gc_new (drawing_area->window);
   gdouble color[4];
   gint8 *dash_list;
   
@@ -656,22 +655,42 @@ get_focus_gc (GtkWidget *drawing_area,
 			NULL);
       
   palette_get_color (drawing_area, color);
-      
-  if (INTENSITY (color[0], color[1], color[2]) > 0.5)
-    gdk_gc_copy (gc, drawing_area->style->black_gc);
-  else
-    gdk_gc_copy (gc, drawing_area->style->white_gc);
 
-  gdk_gc_set_line_attributes (gc, *focus_width,
-			      dash_list[0] ? GDK_LINE_ON_OFF_DASH : GDK_LINE_SOLID,
-			      GDK_CAP_BUTT, GDK_JOIN_MITER);
+  if (INTENSITY (color[0], color[1], color[2]) > 0.5)
+    cairo_set_source_rgb (cr, 0., 0., 0.);
+  else
+    cairo_set_source_rgb (cr, 1., 1., 1.);
+
+  cairo_set_line_width (cr, *focus_width);
 
   if (dash_list[0])
-    gdk_gc_set_dashes (gc, 0, dash_list, strlen ((char *)dash_list));
+    {
+      gint n_dashes = strlen (dash_list);
+      gdouble *dashes = g_new (gdouble, n_dashes);
+      gdouble total_length = 0;
+      gdouble dash_offset;
+      gint i;
+
+      for (i = 0; i < n_dashes; i++)
+	{
+	  dashes[i] = dash_list[i];
+	  total_length += dash_list[i];
+	}
+
+      /* The dash offset here aligns the pattern to integer pixels
+       * by starting the dash at the right side of the left border
+       * Negative dash offsets in cairo don't work
+       * (https://bugs.freedesktop.org/show_bug.cgi?id=2729)
+       */
+      dash_offset = - *focus_width / 2.;
+      while (dash_offset < 0)
+	dash_offset += total_length;
+      
+      cairo_set_dash (cr, dashes, n_dashes, dash_offset);
+      g_free (dashes);
+    }
 
   g_free (dash_list);
-  
-  return gc;
 }
 
 static void
@@ -1160,24 +1179,34 @@ static GdkCursor *
 make_picker_cursor (GdkScreen *screen)
 {
   GdkCursor *cursor;
-  GdkColor bg = { 0, 0xffff, 0xffff, 0xffff };
-  GdkColor fg = { 0, 0x0000, 0x0000, 0x0000 };
-  GdkWindow *window = gdk_screen_get_root_window (screen);
-  
-  GdkPixmap *pixmap =
-    gdk_bitmap_create_from_data (window, (gchar *) dropper_bits,
-				 DROPPER_WIDTH, DROPPER_HEIGHT);
 
-  GdkPixmap *mask =
-    gdk_bitmap_create_from_data (window, (gchar *) dropper_mask,
-				 DROPPER_WIDTH, DROPPER_HEIGHT);
+  cursor = gdk_cursor_new_from_name (gdk_screen_get_display (screen),
+				     "color-picker");
 
-  cursor = gdk_cursor_new_from_pixmap (pixmap, mask, &fg, &bg,
-				       DROPPER_X_HOT, DROPPER_Y_HOT);
-  
-  g_object_unref (pixmap);
-  g_object_unref (mask);
+  if (!cursor)
+    {
+      GdkColor bg = { 0, 0xffff, 0xffff, 0xffff };
+      GdkColor fg = { 0, 0x0000, 0x0000, 0x0000 };
+      GdkWindow *window;
+      GdkPixmap *pixmap, *mask;
 
+      window = gdk_screen_get_root_window (screen);
+      
+      pixmap =
+	gdk_bitmap_create_from_data (window, (gchar *) dropper_bits,
+				     DROPPER_WIDTH, DROPPER_HEIGHT);
+      
+      mask =
+	gdk_bitmap_create_from_data (window, (gchar *) dropper_mask,
+				     DROPPER_WIDTH, DROPPER_HEIGHT);
+      
+      cursor = gdk_cursor_new_from_pixmap (pixmap, mask, &fg, &bg,
+					   DROPPER_X_HOT, DROPPER_Y_HOT);
+      
+      g_object_unref (pixmap);
+      g_object_unref (mask);
+    }
+      
   return cursor;
 }
 
@@ -1223,14 +1252,27 @@ shutdown_eyedropper (GtkWidget *widget)
   GtkColorSelection *colorsel;
   ColorSelectionPrivate *priv;
   GdkDisplay *display = gtk_widget_get_display (widget);
-  guint32 time = gtk_get_current_event_time ();
 
   colorsel = GTK_COLOR_SELECTION (widget);
   priv = colorsel->private_data;    
 
-  gdk_display_keyboard_ungrab (display, time);
-  gdk_display_pointer_ungrab (display, time);
-  gtk_grab_remove (priv->dropper_grab_widget);
+  if (priv->has_grab)
+    {
+      gdk_display_keyboard_ungrab (display, priv->grab_time);
+      gdk_display_pointer_ungrab (display, priv->grab_time);
+      gtk_grab_remove (priv->dropper_grab_widget);
+
+      priv->has_grab = FALSE;
+    }
+}
+
+static gboolean 
+gtk_color_selection_grab_broken (GtkWidget          *widget,
+				 GdkEventGrabBroken *event)
+{
+  shutdown_eyedropper (widget);
+
+  return TRUE;
 }
 
 static void
@@ -1283,10 +1325,13 @@ key_press (GtkWidget   *invisible,
            gpointer     data)
 {  
   GdkDisplay *display = gtk_widget_get_display (invisible);
+  GdkScreen *screen = gdk_event_get_screen ((GdkEvent *)event);
   guint state = event->state & gtk_accelerator_get_default_mod_mask ();
   gint x, y;
   gint dx, dy;
 
+  gdk_display_get_pointer (display, NULL, &x, &y, NULL);
+
   dx = 0;
   dy = 0;
 
@@ -1296,10 +1341,7 @@ key_press (GtkWidget   *invisible,
     case GDK_Return:
     case GDK_KP_Enter:
     case GDK_KP_Space:
-      gdk_display_get_pointer (display, 
-			       NULL, &x, &y, NULL);
-      grab_color_at_mouse (gdk_event_get_screen ((GdkEvent *)event),
-			   x, y, data);
+      grab_color_at_mouse (screen, x, y, data);
       /* fall through */
 
     case GDK_Escape:
@@ -1340,16 +1382,7 @@ key_press (GtkWidget   *invisible,
       return FALSE;
     }
 
-#ifdef GDK_WINDOWING_X11
-  XWarpPointer (gdk_x11_display_get_xdisplay (display),
-		None, None, 0, 0, 0, 0, dx, dy);
-#elif defined GDK_WINDOWING_WIN32
-  {
-    POINT point;
-    if (GetCursorPos (&point))
-      SetCursorPos (point.x + dx, point.y + dy);
-  }
-#endif
+  gdk_display_warp_pointer (display, screen, x + dx, y + dy);
   
   return TRUE;
 
@@ -1394,6 +1427,7 @@ get_screen_color (GtkWidget *button)
   GdkScreen *screen = gtk_widget_get_screen (GTK_WIDGET (button));
   GdkCursor *picker_cursor;
   GdkGrabStatus grab_status;
+  guint32 time = gtk_get_current_event_time ();
   
   if (priv->dropper_grab_widget == NULL)
     {
@@ -1403,11 +1437,14 @@ get_screen_color (GtkWidget *button)
                              GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK);
       
       gtk_widget_show (priv->dropper_grab_widget);
+
+      gdk_window_set_user_data (priv->dropper_grab_widget->window, colorsel);
+      gdk_window_reparent (priv->dropper_grab_widget->window, 
+			   GTK_WIDGET (colorsel)->window, 0, 0);
     }
 
   if (gdk_keyboard_grab (priv->dropper_grab_widget->window,
-                         FALSE,
-                         gtk_get_current_event_time ()) != GDK_GRAB_SUCCESS)
+                         FALSE, time) != GDK_GRAB_SUCCESS)
     return;
   
   picker_cursor = make_picker_cursor (screen);
@@ -1416,16 +1453,18 @@ get_screen_color (GtkWidget *button)
 				  GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK | GDK_POINTER_MOTION_MASK,
 				  NULL,
 				  picker_cursor,
-				  gtk_get_current_event_time ());
+				  time);
   gdk_cursor_unref (picker_cursor);
   
   if (grab_status != GDK_GRAB_SUCCESS)
     {
-      gdk_display_keyboard_ungrab (gtk_widget_get_display (button), GDK_CURRENT_TIME);
+      gdk_display_keyboard_ungrab (gtk_widget_get_display (button), time);
       return;
     }
 
   gtk_grab_add (priv->dropper_grab_widget);
+  priv->grab_time = time;
+  priv->has_grab = TRUE;
   
   g_signal_connect (priv->dropper_grab_widget, "button_press_event",
                     G_CALLBACK (mouse_press), colorsel);
@@ -1666,7 +1705,7 @@ update_color (GtkColorSelection *colorsel)
   gchar *ptr;
   
   priv->changing = TRUE;
-  color_sample_draw_samples (colorsel);
+  color_sample_update_samples (colorsel);
   
   gtk_hsv_set_color (GTK_HSV (priv->triangle_colorsel),
 		     priv->color[COLORSEL_HUE],
@@ -1827,35 +1866,36 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
   widget_class->realize = gtk_color_selection_realize;
   widget_class->unrealize = gtk_color_selection_unrealize;
   widget_class->show_all = gtk_color_selection_show_all;
+  widget_class->grab_broken_event = gtk_color_selection_grab_broken;
   
   g_object_class_install_property (gobject_class,
                                    PROP_HAS_OPACITY_CONTROL,
-                                   g_param_spec_boolean ("has_opacity_control",
+                                   g_param_spec_boolean ("has-opacity-control",
 							 P_("Has Opacity Control"),
 							 P_("Whether the color selector should allow setting opacity"),
 							 FALSE,
-							 G_PARAM_READABLE | G_PARAM_WRITABLE));
+							 GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_HAS_PALETTE,
-                                   g_param_spec_boolean ("has_palette",
+                                   g_param_spec_boolean ("has-palette",
 							 P_("Has palette"),
 							 P_("Whether a palette should be used"),
 							 FALSE,
-							 G_PARAM_READABLE | G_PARAM_WRITABLE));
+							 GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_CURRENT_COLOR,
-                                   g_param_spec_boxed ("current_color",
+                                   g_param_spec_boxed ("current-color",
                                                        P_("Current Color"),
                                                        P_("The current color"),
                                                        GDK_TYPE_COLOR,
-                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                       GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_CURRENT_ALPHA,
-                                   g_param_spec_uint ("current_alpha",
+                                   g_param_spec_uint ("current-alpha",
 						      P_("Current Alpha"),
 						      P_("The current opacity value (0 fully transparent, 65535 fully opaque)"),
 						      0, 65535, 65535,
-						      G_PARAM_READABLE | G_PARAM_WRITABLE));
+						      GTK_PARAM_READWRITE));
   
   color_selection_signals[COLOR_CHANGED] =
     g_signal_new ("color_changed",
@@ -1870,7 +1910,7 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
                                                       P_("Custom palette"),
                                                       P_("Palette to use in the color selector"),
                                                       default_colors,
-                                                      G_PARAM_READWRITE));
+                                                      GTK_PARAM_READWRITE));
 }
 
 /* widget functions */
@@ -2225,7 +2265,7 @@ gtk_color_selection_set_has_opacity_control (GtkColorSelection *colorsel,
 	  gtk_widget_hide (priv->opacity_label);
 	  gtk_widget_hide (priv->opacity_entry);
 	}
-      color_sample_draw_samples (colorsel);
+      color_sample_update_samples (colorsel);
       
       g_object_notify (G_OBJECT (colorsel), "has-opacity-control");
     }
@@ -2465,7 +2505,7 @@ gtk_color_selection_set_previous_color (GtkColorSelection *colorsel,
 		  &priv->old_color[COLORSEL_HUE],
 		  &priv->old_color[COLORSEL_SATURATION],
 		  &priv->old_color[COLORSEL_VALUE]);
-  color_sample_draw_samples (colorsel);
+  color_sample_update_samples (colorsel);
   priv->default_set = TRUE;
   priv->changing = FALSE;
 }
@@ -2489,7 +2529,7 @@ gtk_color_selection_set_previous_alpha (GtkColorSelection *colorsel,
   priv = colorsel->private_data;
   priv->changing = TRUE;
   priv->old_color[COLORSEL_OPACITY] = SCALE (alpha);
-  color_sample_draw_samples (colorsel);
+  color_sample_update_samples (colorsel);
   priv->default_alpha_set = TRUE;
   priv->changing = FALSE;
 }
