@@ -636,7 +636,7 @@ static gchar *
 gtk_file_system_unix_volume_get_display_name (GtkFileSystem       *file_system,
 					      GtkFileSystemVolume *volume)
 {
-  return g_strdup (_("Filesystem")); /* Same as Nautilus */
+  return g_strdup (_("File System")); /* Same as Nautilus */
 }
 
 static IconType
@@ -808,7 +808,7 @@ gtk_file_system_unix_volume_render_icon (GtkFileSystem        *file_system,
 {
   GdkPixbuf *pixbuf;
 
-  pixbuf = get_cached_icon (widget, "gnome-fs-blockdev", pixel_size);
+  pixbuf = get_cached_icon (widget, "gnome-dev-harddisk", pixel_size);
   if (pixbuf)
     return pixbuf;
 
@@ -1224,7 +1224,7 @@ get_icon_type_from_path (GtkFileSystemUnix *system_unix,
 
   icon_type = get_icon_type (filename, NULL);
   if (icon_type == ICON_REGULAR)
-    *mime_type = xdg_mime_get_mime_type_for_file (filename);
+    *mime_type = xdg_mime_get_mime_type_for_file (filename, NULL);
 
   return icon_type;
 }
@@ -1284,6 +1284,9 @@ get_icon_for_mime_type (GtkWidget  *widget,
   GString *icon_name;
   GdkPixbuf *pixbuf;
 
+  if (!mime_type)
+    return NULL;
+
   separator = strchr (mime_type, '/');
   if (!separator)
     return NULL; /* maybe we should return a GError with "invalid MIME-type" */
@@ -1923,7 +1926,7 @@ create_stat_info_entry_and_emit_add (GtkFileFolderUnix *folder_unix,
     entry->statbuf = *statbuf;
 
   if ((folder_unix->types & GTK_FILE_INFO_MIME_TYPE) != 0)
-    entry->mime_type = g_strdup (xdg_mime_get_mime_type_for_file (filename));
+    entry->mime_type = g_strdup (xdg_mime_get_mime_type_for_file (filename, statbuf));
 
   g_hash_table_insert (folder_unix->stat_info,
 		       g_strdup (basename),
@@ -2002,7 +2005,7 @@ gtk_file_folder_unix_get_info (GtkFileFolder      *folder,
 	}
 
       if ((types & GTK_FILE_INFO_MIME_TYPE) != 0)
-	mime_type = xdg_mime_get_mime_type_for_file (filename);
+	mime_type = xdg_mime_get_mime_type_for_file (filename, &statbuf);
       else
 	mime_type = NULL;
 
@@ -2150,13 +2153,17 @@ cb_fill_in_mime_type (gpointer key, gpointer value, gpointer user_data)
   struct stat_info_entry *entry = value;
   GtkFileFolderUnix *folder_unix = user_data;
   char *fullname = g_build_filename (folder_unix->filename, basename, NULL);
+  struct stat *statbuf = NULL;
+  const char *mime_type;
 
-  /* FIXME: Should not need to re-stat.  */
-  const char *mime_type = xdg_mime_get_mime_type_for_file (fullname);
+  if (folder_unix->have_stat)
+    statbuf = &entry->statbuf;
+
+  mime_type = xdg_mime_get_mime_type_for_file (fullname, statbuf);
   entry->mime_type = g_strdup (mime_type);
 
   g_free (fullname);
-  /* FIXME: free on NULL?  */
+
   return FALSE;
 }
 
