@@ -38,12 +38,12 @@
 #ifdef G_OS_WIN32
 #define WIN32_LEAN_AND_MEAN
 #define STRICT
-#include <windows.h>
+#include "gdk/win32/gdkwin32.h"
 #undef STRICT
 #include <shlobj.h>
 #include <shellapi.h>
 #else
-#error "The implementation is win32 only."
+#error "The implementation is Win32 only."
 #endif /* G_OS_WIN32 */
 
 typedef struct _GtkFileSystemWin32Class GtkFileSystemWin32Class;
@@ -291,9 +291,6 @@ check_volumes (gpointer data)
 
   g_return_val_if_fail (system_win32, FALSE);
 
-#if 0
-  printf("check_volumes: system_win32=%p\n", system_win32);
-#endif
   if (system_win32->drives != GetLogicalDrives())
     g_signal_emit_by_name (system_win32, "volumes-changed", 0);
 
@@ -303,10 +300,6 @@ check_volumes (gpointer data)
 static void
 gtk_file_system_win32_init (GtkFileSystemWin32 *system_win32)
 {
-#if 0
-  printf("gtk_file_system_win32_init: %p\n", system_win32);
-#endif
-
   /* set up an idle handler for volume changes, every second should be enough */
   system_win32->timeout = g_timeout_add_full (0, 1000, check_volumes, system_win32, NULL);
 
@@ -320,10 +313,6 @@ gtk_file_system_win32_finalize (GObject *object)
 
   system_win32 = GTK_FILE_SYSTEM_WIN32 (object);
 
-#if 0
-  printf("gtk_file_system_win32_finalize: %p\n", system_win32);
-#endif
-
   g_source_remove (system_win32->timeout);
 
   /* FIXME: assert that the hash is empty? */
@@ -726,7 +715,9 @@ gtk_file_system_win32_get_parent (GtkFileSystem     *file_system,
       *parent = filename_to_path (parent_filename);
       g_free (parent_filename);
     }
-
+#if DEBUGGING_OUTPUT
+  printf ("%s: %s => %s\n", __FUNCTION__, (char*)path, (*parent)?(char*)*parent:"NULL"), fflush(stdout);
+#endif
   return TRUE;
 }
 
@@ -762,8 +753,8 @@ canonicalize_filename (gchar *filename)
   gchar *past_root;
   gboolean last_was_slash = FALSE;
 
-#if 0
-  printf("canonicalize_filename: %s ", filename);
+#if DEBUGGING_OUTPUT
+  printf("%s: %s ", __FUNCTION__, filename), fflush (stdout);
 #endif
 
   past_root = (gchar *) g_path_skip_root (filename);
@@ -775,7 +766,7 @@ canonicalize_filename (gchar *filename)
       if (G_IS_DIR_SEPARATOR (*p))
 	{
 	  if (!last_was_slash)
-	    *q++ = G_DIR_SEPARATOR;
+	    *q++ = '\\';
 
 	  last_was_slash = TRUE;
 	}
@@ -828,8 +819,9 @@ canonicalize_filename (gchar *filename)
     q--;
 
   *q = '\0';
-#if 0
-  printf(" => %s\n", filename);
+
+#if DEBUGGING_OUTPUT
+  printf(" => %s\n", filename), fflush (stdout);
 #endif
 }
 
@@ -845,15 +837,15 @@ gtk_file_system_win32_parse (GtkFileSystem     *file_system,
   gchar *last_backslash, *last_slash;
   gboolean result = FALSE;
 
-#if 0
-  printf("gtk_file_system_win32_parse: base_path=%s str=%s\n",(char*)base_path,str);
+#if DEBUGGING_OUTPUT
+  printf("%s: base_path=%s str=%s\n",__FUNCTION__,(char*)base_path,str),fflush(stdout);
 #endif
 
   base_filename = gtk_file_path_get_string (base_path);
   g_return_val_if_fail (base_filename != NULL, FALSE);
   g_return_val_if_fail (g_path_is_absolute (base_filename), FALSE);
   
-  last_backslash = strrchr (str, G_DIR_SEPARATOR);
+  last_backslash = strrchr (str, '\\');
   last_slash = strrchr (str, '/');
   if (last_slash == NULL ||
       (last_backslash != NULL && last_backslash > last_slash))
@@ -874,18 +866,25 @@ gtk_file_system_win32_parse (GtkFileSystem     *file_system,
 	{
 	  if (g_ascii_isalpha (base_filename[0]) &&
 	      base_filename[1] == ':')
-	    folder_part = g_strdup_printf ("%c:" G_DIR_SEPARATOR_S, base_filename[0]);
+	    folder_part = g_strdup_printf ("%c:\\", base_filename[0]);
 	  else
-	    folder_part = g_strdup (G_DIR_SEPARATOR_S);
+	    folder_part = g_strdup ("\\");
 	}
       else if (g_ascii_isalpha (str[0]) &&
 	       str[1] == ':' &&
-	       G_IS_DIR_SEPARATOR (str[2]))
+	       last_slash == str + 2)
 	folder_part = g_strndup (str, last_slash - str + 1);
+#if 0
+      /* Hmm, what the heck was this case supposed to do? It splits up
+       * \\server\share\foo\bar into folder_part
+       * \\server\share\foo\bar and file_path bar. Not good. As far as
+       * I can see, this isn't needed.
+       */
       else if (G_IS_DIR_SEPARATOR (str[0]) &&
 	       G_IS_DIR_SEPARATOR (str[1]) &&
 	       (!str[2] || !G_IS_DIR_SEPARATOR (str[2])))
 	folder_part = g_strdup (str);
+#endif
       else
 	folder_part = g_strndup (str, last_slash - str);
 
@@ -909,8 +908,8 @@ gtk_file_system_win32_parse (GtkFileSystem     *file_system,
       result = TRUE;
     }
 
-#if 0
-  printf("gtk_file_system_win32_parse:returning folder=%s file_part=%s\n",(*folder?(char*)*folder:"NULL"),*file_part);
+#if DEBUGGING_OUTPUT
+  printf ("%s:returning folder=%s file_part=%s\n", __FUNCTION__, (*folder?(char*)*folder:"NULL"), *file_part), fflush(stdout);
 #endif
 
   return result;
@@ -936,6 +935,11 @@ gtk_file_system_win32_uri_to_path (GtkFileSystem     *file_system,
 {
   GtkFilePath *path = NULL;
   gchar *filename = g_filename_from_uri (uri, NULL, NULL);
+
+#if DEBUGGING_OUTPUT
+  printf ("%s: %s -> %s\n", __FUNCTION__, uri, filename?filename:"NULL"), fflush (stdout);
+#endif
+
   if (filename)
     {
       path = filename_to_path (filename);
@@ -1042,7 +1046,6 @@ extract_icon (const char* filename)
 {
   GdkPixbuf *pixbuf = NULL;
   HICON hicon;
-  ICONINFO ii;
   
   if (!filename || !filename[0])
     return NULL;
@@ -1125,79 +1128,7 @@ extract_icon (const char* filename)
     }
 #endif
   
-  if (GetIconInfo (hicon, &ii))
-    {
-      struct
-      {
-	BITMAPINFOHEADER bi;
-	RGBQUAD colors[2];
-      } bmi;
-      HDC hdc;
-      
-      memset (&bmi, 0, sizeof (bmi));
-      bmi.bi.biSize = sizeof (bmi.bi);
-      hdc = CreateCompatibleDC (NULL);
-      
-      if (GetDIBits (hdc, ii.hbmColor, 0, 1, NULL, (BITMAPINFO *)&bmi, DIB_RGB_COLORS))
-	{
-	  gchar *pixels, *bits;
-	  gint rowstride, x, y, w = bmi.bi.biWidth, h = bmi.bi.biHeight;
-	  gboolean no_alpha;
-	  
-	  bmi.bi.biBitCount = 32;
-	  bmi.bi.biCompression = BI_RGB;
-	  bmi.bi.biHeight = -h;
-	  pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, w, h);
-	  bits = g_malloc0 (4 * w * h);
-	  
-	  /* color data */
-	  if (!GetDIBits (hdc, ii.hbmColor, 0, h, bits, (BITMAPINFO *)&bmi, DIB_RGB_COLORS))
-	    g_warning (G_STRLOC ": Failed to get dibits");
-	  
-	  pixels = gdk_pixbuf_get_pixels (pixbuf);
-	  rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-	  no_alpha = TRUE;
-	  for (y = 0; y < h; y++)
-	    {
-	      for (x = 0; x < w; x++)
-		{
-		  pixels[2] = bits[(x+y*w) * 4];
-		  pixels[1] = bits[(x+y*w) * 4 + 1];
-		  pixels[0] = bits[(x+y*w) * 4 + 2];
-		  pixels[3] = bits[(x+y*w) * 4 + 3];
-		  if (no_alpha && pixels[3] > 0) no_alpha = FALSE;
-		  pixels += 4;
-		}
-	      pixels += (w * 4 - rowstride);
-	    }
-	  /* mask */
-	  if (no_alpha) {
-	    if (!GetDIBits (hdc, ii.hbmMask, 0, h, bits, (BITMAPINFO *)&bmi, DIB_RGB_COLORS))
-	      g_warning (G_STRLOC ": Failed to get dibits");
-	    pixels = gdk_pixbuf_get_pixels (pixbuf);
-	    for (y = 0; y < h; y++)
-	      {
-		for (x = 0; x < w; x++)
-		  {
-		    pixels[3] = 255 - bits[(x + y * w) * 4];
-		    pixels += 4;
-		  }
-		pixels += (w * 4 - rowstride);
-	      }
-	    
-	    /* release temporary resources */
-	    g_free (bits);
-	    if (!DeleteObject (ii.hbmColor) || !DeleteObject (ii.hbmMask))
-	      g_warning (G_STRLOC ": Leaking Icon Bitmaps ?");
-	  }
-	}
-      else
-	g_warning (G_STRLOC ": GetDIBits () failed, %s", g_win32_error_message (GetLastError ()));
-      
-      DeleteDC (hdc);
-    }
-  else
-    g_warning (G_STRLOC ": GetIconInfo failed: %s\n", g_win32_error_message (GetLastError ())); 
+  pixbuf = gdk_win32_icon_to_pixbuf_libgtk_only (hicon);
   
   if (!DestroyIcon (hicon))
     g_warning (G_STRLOC ": DestroyIcon failed: %s\n", g_win32_error_message (GetLastError ()));
@@ -1495,6 +1426,11 @@ gtk_file_folder_win32_list_children (GtkFileFolder  *folder,
   *children = NULL;
 
   dir = g_dir_open (folder_win32->filename, 0, error);
+
+#if DEBUGGING_OUTPUT
+  printf ("%s: %s: %s\n", __FUNCTION__, folder_win32->filename, dir?"OK":"FAIL"), fflush (stdout);
+#endif
+
   if (!dir)
     return FALSE;
 
@@ -1589,9 +1525,7 @@ filename_get_info (const gchar     *filename,
       
       if (types & GTK_FILE_INFO_IS_HIDDEN)
 	{
-	  /* Unix dot convention or the Windows hidden attribute */
-	  gboolean is_hidden = basename[0] == '.' ||
-	    !!(wfad.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN);
+	  gboolean is_hidden = !!(wfad.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN);
 	  gtk_file_info_set_is_hidden (info, is_hidden);
 	}
 
@@ -1613,23 +1547,37 @@ filename_get_info (const gchar     *filename,
     }
 #endif
 
-  if ((types & GTK_FILE_INFO_MIME_TYPE)
-#if 0 /* it's dead in gtkfilesystemunix.c, too */
-      || ((types & GTK_FILE_INFO_ICON) && icon_type == GTK_FILE_ICON_REGULAR)
-#endif
-     )
+  if (types & GTK_FILE_INFO_MIME_TYPE)
     {
-#if 0
-      const char *mime_type = xdg_mime_get_mime_type_for_file (filename);
-      gtk_file_info_set_mime_type (info, mime_type);
+      if (g_file_test (filename, G_FILE_TEST_IS_EXECUTABLE))
+	gtk_file_info_set_mime_type (info, "application/x-executable");
+      else
+	{
+	  const char *extension = strrchr (filename, '.');
+	  HKEY key = NULL;
+	  DWORD type, nbytes = 0;
+	  char *value;
 
-      if ((types & GTK_FILE_INFO_ICON) && icon_type == GTK_FILE_ICON_REGULAR &&
-	  (statbuf.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) &&
-	  (strcmp (mime_type, XDG_MIME_TYPE_UNKNOWN) == 0 ||
-	   strcmp (mime_type, "application/x-executable") == 0 ||
-	   strcmp (mime_type, "application/x-shellscript") == 0))
-	gtk_file_info_set_icon_type (info, GTK_FILE_ICON_EXECUTABLE);
-#endif
+	  if (extension != NULL &&
+	      extension[1] != '\0' &&
+	      RegOpenKeyEx (HKEY_CLASSES_ROOT, extension, 0,
+			    KEY_QUERY_VALUE, &key) == ERROR_SUCCESS &&
+	      RegQueryValueEx (key, "Content Type", 0,
+			       &type, NULL, &nbytes) == ERROR_SUCCESS &&
+	      type == REG_SZ &&
+	      (value = g_try_malloc (nbytes + 1)) &&
+	      RegQueryValueEx (key, "Content Type", 0,
+			       &type, value, &nbytes) == ERROR_SUCCESS)
+	    {
+	      value[nbytes] = '\0';
+	      gtk_file_info_set_mime_type (info, value);
+	      g_free (value);
+	    }
+	  else
+	    gtk_file_info_set_mime_type (info, "application/octet-stream");
+	  if (key != NULL)
+	    RegCloseKey (key);
+	}
     }
 
   if (types & GTK_FILE_INFO_MODIFICATION_TIME)
@@ -1713,14 +1661,8 @@ filename_is_server_share (const char *filename)
 static gboolean
 filename_is_some_root (const char *filename)
 {
-#if 0
-  return ((G_IS_DIR_SEPARATOR (filename[0]) && filename[1] == '\0') ||
-	  filename_is_server_share (filename) ||
-	  filename_is_drive_root (filename));
-#else
   return (g_path_is_absolute (filename) &&
 	  *(g_path_skip_root (filename)) == '\0');
-#endif
 }    
 
 int
